diff -ruN kernel/wctdm24xxp.orig/base.c kernel/wctdm24xxp/base.c
--- kernel/wctdm24xxp.orig/base.c	2008-04-11 08:07:59.000000000 -0300
+++ kernel/wctdm24xxp/base.c	2008-04-11 08:46:34.000000000 -0300
@@ -301,6 +301,7 @@
 /* See vpmnlptype = 4 for more info */
 static int vpmnlpmaxsupp = 0;
 #endif
+static int dtmf = 0;
 
 static int wctdm_init_proslic(struct wctdm *wc, int card, int fast , int manual, int sane);
 
@@ -1038,6 +1039,29 @@
 		cmd_vpm150m_retransmit(wc);
 #endif
 }
+static inline void wctdm_dtmfcheck_fakepolarity(struct wctdm *wc, int card, int x) 
+{
+	int sample;
+    /* only look for sound on the line if dtmf flag is on, it is an fxo card and line is onhook */ 
+	if (!dtmf || !(wc->cardflag & (1 << card)) || !(wc->modtype[card] == MOD_TYPE_FXO) || wc->mods[card].fxo.offhook ) 
+        return;
+
+    /* don't look for noise if we're already processing it, or there is a ringing tone */
+	if(!wc->mods[card].fxo.readcid && !wc->mods[card].fxo.wasringing  &&
+		wc->intcount > wc->mods[card].fxo.cidtimer + 400 ) {
+		sample = ZT_XLAW(wc->chans[card].readchunk[x], (&(wc->chans[card])));
+		if (sample > 16000 || sample < -16000) {
+			wc->mods[card].fxo.readcid = 1;
+			wc->mods[card].fxo.cidtimer = wc->intcount;
+			if (debug) printk("DTMF CLIP on %i\n",card+1);
+			zt_qevent_lock(&wc->chans[card], ZT_EVENT_POLARITY);
+		}
+	} else if(wc->mods[card].fxo.readcid && wc->intcount > wc->mods[card].fxo.cidtimer + 2000) {
+        /* reset flags if it's been a while */
+		wc->mods[card].fxo.cidtimer = wc->intcount;
+		wc->mods[card].fxo.readcid = 0;
+	}
+}
 
 static inline void wctdm_receiveprep(struct wctdm *wc, int dbl)
 {
@@ -1064,6 +1088,7 @@
 				wc->chans[y].readchunk[x] = readchunk[y];
 			}
 			cmd_decifer(wc, readchunk, y);
+			wctdm_dtmfcheck_fakepolarity(wc,y,x);
 		}
 #ifdef VPM_SUPPORT
 		if (wc->vpm) {
@@ -1358,6 +1383,8 @@
 				} else if (!res) {
 					if ((wc->mods[card].fxo.ringdebounce == 0) && wc->mods[card].fxo.wasringing) {
 						wc->mods[card].fxo.wasringing = 0;
+ 						wc->mods[card].fxo.readcid = 0;
+ 						wc->mods[card].fxo.cidtimer = wc->intcount;
 						if (debug)
 							printk("NO RING on %d/%d!\n", wc->span.spanno, card + 1);
 						zt_hooksig(&wc->chans[card], ZT_RXSIG_OFFHOOK);
@@ -1385,6 +1412,8 @@
 				if (wc->mods[card].fxo.ringdebounce <= 0) {
 					if (wc->mods[card].fxo.wasringing) {
 						wc->mods[card].fxo.wasringing = 0;
+	 					wc->mods[card].fxo.readcid = 0;
+	 					wc->mods[card].fxo.cidtimer = wc->intcount;
 						zt_hooksig(&wc->chans[card], ZT_RXSIG_OFFHOOK);
 						if (debug)
 							printk("NO RING on %d/%d!\n", wc->span.spanno, card + 1);
@@ -4433,6 +4462,7 @@
 module_param(fxsrxgain, int, 0600);
 module_param(ringdebounce, int, 0600);
 module_param(fwringdetect, int, 0600);
+module_param(dtmf, int, 0600);
 #ifdef VPM_SUPPORT
 module_param(vpmsupport, int, 0600);
 module_param(vpmdtmfsupport, int, 0600);
@@ -4462,6 +4492,7 @@
 MODULE_PARM(fxsrxgain, "i");
 MODULE_PARM(ringdebounce, "i");
 MODULE_PARM(fwringdetect, "i");
+MODULE_PARM(dtmf, "i");
 #ifdef VPM_SUPPORT
 MODULE_PARM(vpmsupport, "i");
 MODULE_PARM(vpmdtmfsupport, "i");
diff -ruN kernel/wctdm24xxp.orig/wctdm24xxp.h kernel/wctdm24xxp/wctdm24xxp.h
--- kernel/wctdm24xxp.orig/wctdm24xxp.h	2008-04-11 08:07:59.000000000 -0300
+++ kernel/wctdm24xxp/wctdm24xxp.h	2008-04-11 08:21:22.000000000 -0300
@@ -226,6 +226,8 @@
 			int lastpol;
 			int polarity;
 			int polaritydebounce;
+			int readcid;
+			unsigned int cidtimer;
 		} fxo;
 		struct {
 			int oldrxhook;
